package test.salsadev.data;

import test.salsadev.model.Member;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


@Repository("MemberDAO")
public class MemberDAO {
   
   @PersistenceContext
   private EntityManager em;

   public List<Member> getMembers() {
       CriteriaBuilder cb = em.getCriteriaBuilder();
      CriteriaQuery<Member> criteria = cb.createQuery(Member.class);
      Root<Member> member = criteria.from(Member.class);
      // Swap criteria statements if you would like to try out type-safe criteria queries, a new
      // feature in JPA 2.0
      // criteria.select(member).orderBy(cb.asc(member.get(Member_.name)));
      criteria.select(member).orderBy(cb.asc(member.get("name")));
      return em.createQuery(criteria).getResultList();   
   }

    public Member getById(long id) {
	return em.find(Member.class, id);
    }

    public List<Member> getAllMembers() {
	 return em.createQuery("select m from Member m order by m.name").getResultList();
    }

    @Transactional(propagation= Propagation.REQUIRES_NEW)
    public Member saveMember(Member member) {
	em.persist(member);
	return em.merge(member);
    }
}
