package test.salsadev.rest;

import java.util.ArrayList;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;


import org.apache.lucene.search.Query;
import org.hibernate.search.FullTextQuery;
import org.hibernate.search.jpa.FullTextEntityManager;
import org.hibernate.search.jpa.Search;
import org.hibernate.search.query.dsl.QueryBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import test.salsadev.model.Member;

@Controller
@RequestMapping("/rest/search")
public class MemberResourceRESTSearch {

    public static Logger logger = LoggerFactory.getLogger(MemberResourceRESTSearch.class);
    @PersistenceContext
    private EntityManager em;

    @RequestMapping(value = "/{q}", method = RequestMethod.GET)
    @ResponseBody
    @Transactional
    public List<Member> searchMembers(final @PathVariable String q) {



	List<Member> results = new ArrayList<Member>();


	try {
	    logger.info("SETUP NOW ~~~~~~~~~~~~");
	    final FullTextEntityManager fullTextSession = Search.getFullTextEntityManager(em);


	    final QueryBuilder queryBuilder = fullTextSession.getSearchFactory().buildQueryBuilder().forEntity(Member.class).get();
	    final Query lquery = queryBuilder.keyword().onFields("name", "email").matching(q).createQuery();
	    final org.hibernate.search.jpa.FullTextQuery query = fullTextSession.createFullTextQuery(lquery, Member.class);



	    logger.info("SEARCHING NOW ~~~~~~~~~~~~");

	    //Criteria criteria = ((Session)entityManager.getDelegate()).createCriteria(Document.class);
	    //criteria.add(Restrictions.eqProperty("author", "damien"));


	    //Execute the search now. 
	    query.setFirstResult(0);
	    query.setMaxResults(10);
	    int total = query.getResultSize();

	    //get the results
	    query.setProjection(FullTextQuery.SCORE, FullTextQuery.THIS);
	    List<Object[]> lresults = query.getResultList();
	    for (Object[] res : lresults) {
		float score = (Float) res[0];
		Member member = (Member) res[1];
		logger.info("RESULT score: " + score + " -- " + member.getName());
		results.add(member);
	    }


	} catch (Exception e) {
	    logger.error(e.getMessage(), e);
	}
	return results;

    }
}
